/**
 * @class CQ.wcm.msm.CloudwordsProject
 * @extends CQ.Ext.Panel
 * This provides a panel to see project pages and get project information
 * @constructor
 * Creates a new CloudwordsProject.
 * @param {Object} config The config object
 */
CQ.wcm.msm.CloudwordsProject = CQ.Ext.extend(CQ.Ext.Panel, {
    /**
     * @cfg {String} url
     * URL to retrieve the CloudwordsSource store.
     */
    url: null,

    /**
     * @cfg {String} rootPath
     * Path of the Blueprint
     */
    rootPath: null,

    contextMenus: [],

    constructor: function(config) {
        config = (!config ? {} : config);
        config = CQ.Util.applyDefaults(config, {
            //"url":"/libs/wcm/msm/content/commands/blueprintstatus.json",
        	//url:"/etc/cloudwords/util.projectPageTree.js",
            rootPath:"/content",
            defaultLCFilter:""
        });

        var currentObj = this;
        
        this.url = config.url;
        this.rootPath = config.rootPath;
        this.lcMaxColumns = CQ.wcm.msm.CloudwordsSource.DEFAULT_MAX_VISIBLE_COLUMNS;
        //this.headerStore = this.getHeaderStore(this.url, this.rootPath, config.defaultLCFilter);
        this.sourceContainer = new CQ.Ext.Panel({
        	"title": "Source Project Pages",
            "region": "west",
            "split": true,
            "layout":"fit",
            "collapsible": true,
            "collapseMode":"mini",
            "animate": true,
            "hideCollapseTool": true,
            "border": false,
            "width": 557
        });

        // make the inner label for the project container
        this.projectInfoLabel = new CQ.Ext.form.Label({
			"text": "Loading...",
			"border": false,
			"region": "center"
		});
        
        // make go to cloudwords button
        this.goToCloudwordsButton = new CQ.Ext.Button({
			"text": "View project in Cloudwords",
			"handler": function(b,e) {
				currentObj.goToCloudwordsProject();
			},
			"cls": "cw-info-link",
			"padding": "15px",
			"region": "south",
			"disabled": true
		});
        
        this.projectContainer = new CQ.Ext.Panel({
        	"title": "Project Information",
            "region": "north",
            "height": 250,
            "layout": "border",
            "border": false,
            "items": [ this.projectInfoLabel ],
            "tbar": [ 
                {xtype: 'tbfill'}, // or '->'
                this.goToCloudwordsButton 
            ],
            "bodyStyle": {"background-color":"white"}
            //,
            //"margins": "0 5 0 5"
        });
        


        this.currentPageContainer = new CQ.Ext.Panel({
        	layout: "fit",
    		title: "Selected Page",
    		region: "center",
        	height: 321,
        	border: false
        });

        this.rightSideContainer = new CQ.Ext.Panel({
        	region: "center",
        	border: false,
        	layout: "border",
        	items: [
        		this.projectContainer,
        		this.currentPageContainer
        	]
        });

        var defaults = {
            "layout": "border",
            "height": 600,
            "width": 900,
            //default tree config
            "tree": {
                "lines": true,
                "border": false,
                "borderWidth": CQ.Ext.isBorderBox ? 0 : 2, // the combined left/right border for each cell
                "cls": "x-column-tree",
                "stateful":false,
                "rootVisible": false,
                "autoScroll":true
            },
            "form": {
                "border": false
            },
            "items": [
                this.sourceContainer,
                this.rightSideContainer
            ]
        };

        CQ.Util.applyDefaults(config, defaults);

        // init component by calling super constructor
        CQ.wcm.msm.CloudwordsDeploy.superclass.constructor.call(this, config);
    },

    getGrid: function() {

    	var currentObj = this;

    	var sourcePathStore = new CQ.Ext.data.JsonStore({
    		// store configs
    		autoDestroy: true,
    		autoLoad: true,
    		storeId: "sourcePathStore",
    		proxy: new CQ.Ext.data.HttpProxy({
    			method: "GET",
    			url: CQ.wcm.msm.CloudwordsProject.PROJECT_INFO_URL+ this.url
    		}),
    		// reader configs
    		root: "pages",
    		idProperty: "path",
    		fields: ["path", "title"]
    	});

        this.grid = new CQ.Ext.grid.GridPanel(CQ.Util.applyDefaults({
            "id": "cq-cloudwords-grid",
            //"height": 350,
            "cm":new CQ.Ext.grid.ColumnModel({
            	"columns": [
            	    {"id": "title", "header": "Title","width": 200, "sortable":true,dataIndex: "title"},
            	    {"id": "path", "header": "Path","width": 340, "sortable":true,dataIndex: "path"}
    	        ]
            }),
            "border": false,
            "viewConfig": new CQ.Ext.grid.GridView({}),
            "store": sourcePathStore,
            "listeners": {
            	"rowclick": function(theGrid, rowIndex, e) {
            		currentObj.currentPagePane.removeAll();
            		var row = theGrid.getSelectionModel().getSelected();
            		if(row) {
            			currentObj.updateCurrentPagePane(currentObj.currentPagePane,
            					row, currentObj.projectInfo.status);
            		}
            	}
            }
        }, this.initialConfig["grid"]));

        return this.grid;
    },

    getProjectPane: function() {

    	this.projectPane = new CQ.Ext.Panel({
        	region: "center",
        	border: false

        });
    	return this.projectPane;
    },

    getCurrentPagePane: function() {
    	this.currentPagePane = new CQ.Ext.Panel({
        	region: "center",
        	border: false
        });
    	return this.currentPagePane;
    },
    
    updateCurrentPagePane: function(pagePane, row, status) {
    	//console.log("selected: "+row.json.title);
    	// build the HTML for the Selected Page panel
    	var html = "<div class='selected-page'><p><span class='cloudwords-title'>";
    	html += row.json.title;
    	html += "</span></p>";
    	html += "<p><span class='cloudwords-targetlangs'>Target Languages:</span></p>";
    	var targets = row.json.targets;
    	for(var lang in targets) {
    		if(targets.hasOwnProperty(lang)) {
            	if("In Review" === status) {
            		html += "<span class='cloudwords-preview'><a href='";
                	html += targetPath;
                	html += ".preview.html'>Preview?</a></span>";
            	}
    			html += "<p><span class='cloudwords-language'>";
            	var targetPath = targets[lang];
            	//console.log(JSON.stringify(targetPath));
            	html += lang;
            	html += "</span><br/>";
    			html += "<a href='" + targetPath + ".html'>"
            	html += targetPath;
            	html += "</a></p>";
            	
    		}
    	}
    	html += "</div>";
    	
    	pagePane.add(new CQ.Ext.form.Label({
			"html": html,
			"border": false
		}));
    	pagePane.doLayout();
    },

    build: function() {
    	var myGrid = this.getGrid();
    	//var myProject = this.getProjectPane();
    	var myPage = this.getCurrentPagePane();

    	this.sourceContainer.add(myGrid);
    	//this.projectContainer.add(myProject);
    	this.currentPageContainer.add(myPage);

    	
    	CQ.Ext.Ajax.request({
    		url: CQ.wcm.msm.CloudwordsProject.PROJECT_INFO_URL+this.url,
    		success: function(resp) {
    			this.projectInfo = CQ.Ext.decode(resp.responseText);
    			var info = [];
    			info.push(['Project Name: ',this.projectInfo.cloudwordsName]);
    			info.push(['Source Language: ',this.projectInfo.baseLanguage]);
    			info.push(['Status: ',this.projectInfo.status]);

    			// build dumb html table
    			var html = "<table class='cw-project-info' style='padding:5px;'>";
    			for(var i = 0; i < info.length; i++) {
    				html+="<tr><td class='cw-info-label' width='120' style='padding-right:5px;'>";
    				html+= info[i][0];
    				html+="</td><td class='cw-info-value'>";
    				html+= info[i][1];
    				html+="</td></tr>";
    			}
    			html+= "</table>";
    			
    			// replace the project info label html
    			this.projectInfoLabel.setText(html,false);
    			
    			// enable the cloudwords button now that we have the project info
    			this.goToCloudwordsButton.setDisabled(false);
    			
    			this.projectContainer.doLayout();
    		},
    		failure: function() {
    			// dunno what to do if it fails, guess they won't get info
    			// the user probably should restart
    		},
    		scope: this
    	});
    },
    
    goToCloudwordsProject: function(url) {
    	window.location = this.projectInfo.cloudwordsUrl;
    },

    reconfigure: function(rootPath) {
        this.rootPath = rootPath;
        //this.tree.reload();
        this.tree.reconfigure(this.rootPath);
        //this.redrawForm();
    },

    // overriding CQ.Ext.Component#initComponent
    initComponent: function() {
      CQ.wcm.msm.CloudwordsProject.superclass.initComponent.call(this);
//        this.on("render", function() {
//            this.headerStore.load({
//                "callback": this.build,
//                "scope": this})
//        }, this);
      this.build();
    }
});


CQ.wcm.msm.CloudwordsProject.PROJECT_INFO_URL = "/etc/cloudwords/util.projectPageInfo.js?projectPath=";
CQ.wcm.msm.CloudwordsProject.PROJECT_GRID_URL = "/etc/cloudwords/util.projectPageGrid.js?projectPath=";



