/**
 * @class CQ.wcm.msm.CloudwordsProjects
 * @extends CQ.Ext.Panel
 * This provides a panel to see project pages and get project information
 * @constructor
 * Creates a new CloudwordsProjects.
 * @param {Object} config The config object
 */
CQ.wcm.msm.CloudwordsProjects = CQ.Ext.extend(CQ.Ext.Panel, {
    /**
     * @cfg {String} url
     * URL to retrieve the CloudwordsSource store.
     */
    url: null,

    /**
     * @cfg {String} rootPath
     * Path of the Blueprint
     */
    rootPath: null,

    contextMenus: [],

    constructor: function(config) {
        config = (!config ? {} : config);
        config = CQ.Util.applyDefaults(config, {
            //"url":"/libs/wcm/msm/content/commands/blueprintstatus.json",
        	url:"/etc/cloudwords/util.projects.js",
            rootPath:"/content",
            defaultLCFilter:""
        });

        this.url = config.url;
        this.rootPath = config.rootPath;
        this.lcMaxColumns = CQ.wcm.msm.CloudwordsSource.DEFAULT_MAX_VISIBLE_COLUMNS;
        this.projectsStore = this.getProjectsStore();
        this.projectsContainer = new CQ.Ext.Panel({
            // NOTE: region should be west if right side is there
        	//"region": "west",
        	region: "center",
            "split": true,
            "layout":"fit",
            "collapsible": true,
            "collapseMode":"mini",
            "animate": true,
            "hideCollapseTool": true,
            "border": false,
            "width": 557
        });

        var currentObj = this;

        var defaults = {
            "layout": "border",
            "height": 300,
            "width": 650,
            //default tree config
            "tree": {
                "lines": true,
                "border": false,
                "borderWidth": CQ.Ext.isBorderBox ? 0 : 2, // the combined left/right border for each cell
                "cls": "x-column-tree",
                "stateful":false,
                "rootVisible": false,
                "autoScroll":true
            },
            "form": {
                "border": false
            },
            "items": [
                this.projectsContainer//,
                //this.rightSideContainer
            ]
        };

        CQ.Util.applyDefaults(config, defaults);

        // init component by calling super constructor
        CQ.wcm.msm.CloudwordsDeploy.superclass.constructor.call(this, config);
    },
    
    getProjectsStore: function() {
        return new CQ.Ext.data.JsonStore({
    		// store configs
    		autoDestroy: true,
    		autoLoad: true,
    		storeId: "projectsStore",
    		proxy: new CQ.Ext.data.HttpProxy({
    			method: "GET",
    			url: this.url
    		}),
    		// reader configs
    		root: "projects",
    		idProperty: "cloudwordsId",
    		fields: [
    		    "name", 
    		    "cloudwordsName",
    		    "status", 
    		    "sourceLanguage", 
    		    "targetLanguages",
    		    "cloudwordsId"
    		]
    	});
    },

    getGrid: function() {

    	var currentObj = this;

        this.grid = new CQ.Ext.grid.GridPanel(CQ.Util.applyDefaults({
            "id": "cq-cloudwords-grid",
            //"height": 350,
            "cm":new CQ.Ext.grid.ColumnModel({
            	"columns": [
            	    {"id": "cloudwordsName", "header": "Name","width": 175, "sortable":true,dataIndex: "cloudwordsName"},
    	            {"id": "sourceLang", "header": "Source Language","width": 100, "sortable":true,dataIndex: "sourceLanguage"},
    	            {"id": "targetLangs", "header": "Target Languages","width": 100, "sortable":true,dataIndex: "targetLanguages"},
    	            {"id": "status", "header": "Status","width": 160, "sortable":true,dataIndex: "status"}
    	        ]
            }),
            "border": false,
            "viewConfig": new CQ.Ext.grid.GridView({}),
            "store": this.projectsStore,
            "listeners": {
            	"rowdblclick": function(theGrid, index, e) {
            		var row = theGrid.getSelectionModel().getSelected();
            		//alert(JSON.stringify(row));
                    window.location = "/etc/cloudwords/projects/"+row.json.name+".html";
                }
            }
        }, this.initialConfig["grid"]));

        return this.grid;
    },

    getProjectPane: function() {
    	this.projectInfoValues = {};
    	this.projectInfoValues.name = new CQ.Ext.form.Label({
    		text: 'Project Name'
    	});
    	this.projectInfoValues.sourceLang = new CQ.Ext.form.Label({
    		text: 'Project Name'
    	});
    	this.projectInfoValues.basePath = new CQ.Ext.form.Label({
    		text: 'Project Name'
    	});
    	this.projectInfoValues.targetLangs = new CQ.Ext.Panel({
    		border: false,
    		layout: 'fit'
    	});

    	this.projectPane = new CQ.Ext.Panel({
        	region: "center",
        	border: false
        });
    	return this.projectPane;
    },

    getCurrentPagePane: function() {
    	this.currentPagePane = new CQ.Ext.Panel({
        	region: "center",
        	border: false
        });
    	return this.currentPagePane;
    },

    build: function() {
    	var myGrid = this.getGrid();
    	//var myProject = this.getProjectPane();
    	var myPage = this.getCurrentPagePane();

    	this.projectsContainer.add(myGrid);
    	//this.projectContainer.add(myProject);
    	//this.currentPageContainer.add(myPage);
    	//this.projectsStore.load();
    	this.doLayout();
    },

    reconfigure: function(rootPath) {
        this.rootPath = rootPath;
        //this.tree.reload();
        this.tree.reconfigure(this.rootPath);
        //this.redrawForm();
    },

    // overriding CQ.Ext.Component#initComponent
    initComponent: function() {
        CQ.wcm.msm.CloudwordsProject.superclass.initComponent.call(this);
        this.on("render", function() {
            this.projectsStore.load({
                "callback": this.build,
                "scope": this})
        }, this);
    }
});

CQ.Ext.reg("cloudwordsprojects", CQ.wcm.msm.CloudwordsProjects);